@@ -38,20 +38,15 @@ GPlugin *GPluginLoader::load(const QString &strName,
   InitPlugin init;
   GPluginSpec spec;
 
-  spec.lib = new QLibrary("libgnunetqtmodule_" + strName);
+  spec.lib = GNUNET_plugin_load(errorContext, "libgnunetqtmodule_", qPrintable(strName));
   spec.wnd = NULL;
 
-  // TODO: error handling, see Jean Michault's mail
-
   if (spec.lib)
   {
     plugins.append(spec);
-    init = (InitPlugin) spec.lib->resolve("init_plugin");
+    init = (InitPlugin) GNUNET_plugin_resolve_function(spec.lib, "init_", GNUNET_YES);
     if (init)
       spec.wnd = init(config, errorContext);
-    else
-      GNUNET_GE_LOG(errorContext, (GNUNET_GE_KIND) (GNUNET_GE_FATAL | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
-        qPrintable(spec.lib->errorString()));
   }
     
   return spec.wnd;
@@ -66,11 +61,11 @@ void GPluginLoader::unloadAll()
   while (count)
   {
     spec = plugins.takeLast();
-    shutdown = (ShutdownPlugin) spec.lib->resolve("shutdown_plugin");
+    shutdown = (ShutdownPlugin) GNUNET_plugin_resolve_function(spec.lib, "shutdown_", GNUNET_YES);
     if (shutdown)
       shutdown(spec.wnd);
     
-    delete spec.lib;
+    GNUNET_plugin_unload(spec.lib);
     count--;
   }
 }
