@@ -28,11 +28,34 @@
 #define PLUGINLOADER_H_
 
 #include <QObject>
-#include <QLibrary>
+#include <GNUnet/gnunet_util.h>
 
-#include "gnunet_qt_common.h"
+#include "plugin.h"
+
+typedef struct
+{
+  struct GNUNET_PluginHandle *lib;
+  class GPlugin *wnd;
+} GPluginSpec;
 
 
+class GPluginLoader : public QObject
+{
+  Q_OBJECT
+  
+public:
+  ~GPluginLoader();
+  GPlugin *load(const QString &strName, struct GNUNET_GC_Configuration *config,
+    struct GNUNET_GE_Context *errorContext);
+  void unloadAll();
+  
+protected:
+  QList<GPluginSpec> plugins;
+};
+
+typedef GPlugin *(*InitPlugin) (const struct GNUNET_GC_Configuration *config,
+  const struct GNUNET_GE_Context *errorContext);
+typedef void (*ShutdownPlugin) (GPlugin *plugin);
 
 #endif /*PLUGINLOADER_H_*/
 
