@@ -85,7 +85,7 @@ static struct GNUNET_CommandLineOption gnunetqtOptions[] = {
     LCID lcid;
     char locale[6];
   } GLcid2Locale;
-  
+
   static const GLcid2Locale Lcid2Locale[] =
   {
     { 0x0436,"af_ZA"},  { 0x041c,"sq_AL"},  { 0x045e,"am_ET"},  { 0x1401,"ar_DZ"},
@@ -131,20 +131,20 @@ static struct GNUNET_CommandLineOption gnunetqtOptions[] = {
     { 0x042a,"vi_VN"},  { 0x0452,"cy_GB"},  { 0x0488,"wo_SN"},  { 0x0434,"xh_ZA"},
     { 0x0478,"ii_CN"},  { 0x046a,"yo_NG"},  { 0x0435,"zu_ZA"},  { 0x0000,"C"}
   };
-  
+
   const char *locale()
   {
     GLcid2Locale *entry = (GLcid2Locale *) Lcid2Locale;
     LCID lcid = GetThreadLocale() & 0x7FFF;
-    
+
     while(entry->lcid)
     {
       if (entry->lcid == lcid)
         return entry->locale;
-      
+
       entry++;
     }
-    
+
     return entry->locale;
   }
 #endif
@@ -158,9 +158,9 @@ typedef struct
 static void logger(void *cls, GNUNET_GE_KIND kind, const char *date, const char *msg)
 {
   Q_UNUSED(date)
-  
+
   QPixmap icon;
-  
+
   if (kind & GNUNET_GE_DEBUG)
     icon.load(":/pixmaps/debug.png");
   else if (kind & GNUNET_GE_STATUS)
@@ -173,18 +173,18 @@ static void logger(void *cls, GNUNET_GE_KIND kind, const char *date, const char
     icon.load(":/pixmaps/error.png");
   else if (kind & GNUNET_GE_FATAL)
     icon.load(":/pixmaps/fatal.png");
-    
+
   ((GLoggerClosure *) cls)->lock.lock();
   ((GLoggerClosure *) cls)->mainWnd->setStatusText(icon, QString(msg));
   ((GLoggerClosure *) cls)->lock.unlock();
 }
 
-GApplication::GApplication(int argc, char **argv,
+GApplication::GApplication(int &argc, char **argv,
     struct GNUNET_GC_Configuration *cfg) : QApplication(argc, argv)
 {
   this->ectx = NULL;
   this->cfg = cfg;
-  
+
   strCfgFile = cfgFilename;
 }
 
@@ -193,32 +193,83 @@ void GApplication::setErrorContext(struct GNUNET_GE_Context *ectx)
   this->ectx = ectx;
 }
 
+void GApplication::setupMenuStruct()
+{
+  GMenuAction action;
+  GMenu menu;
+
+  // File menu
+  menu.name = "gnunet-qt::core::file";
+  menu.title = QApplication::translate("WndMain", "&File", 0, QApplication::UnicodeUTF8);
+
+  action.name = "gnunet-qt::core::file::exit";
+  action.icon = QIcon(":/pixmaps/exit.png");
+  action.text = QApplication::translate("WndMain", "&Exit", 0, QApplication::UnicodeUTF8);
+  action.menuRole = QAction::QuitRole;
+  action.receiver = this;
+  action.receiverSlot = SLOT(quit());
+  action.action = &wnd.actionExit;
+  menu.actions.push_back(action);
+
+  menuStruct.push_back(menu);
+
+  //Help menu
+  menu.name = "gnunet-qt::core::help";
+  menu.title = QApplication::translate("WndMain", "&Help", 0, QApplication::UnicodeUTF8);
+  menu.actions.clear();
+
+  action.name = "gnunet-qt::core::help::contextHelp";
+  action.icon = QIcon(":/pixmaps/whatsthis.png");
+  action.text = QApplication::translate("WndMain", "&Context help", 0, QApplication::UnicodeUTF8);
+  action.receiver = &wnd;
+  action.receiverSlot = SLOT(contextHelp());
+  action.action = &wnd.action_Context_help;
+  menu.actions.push_back(action);
+
+  action.name = "gnunet-qt::core::help::about";
+  action.icon = QIcon(":/pixmaps/gnunet-logo-small.png");
+  action.text = QApplication::translate("WndMain", "About", 0, QApplication::UnicodeUTF8);
+  action.menuRole = QAction::AboutRole;
+  action.receiver = &wnd;
+  action.receiverSlot = SLOT(about());
+  action.action = &wnd.actionAbout;
+  menu.actions.push_back(action);
+
+  menuStruct.push_back(menu);
+}
+
 void GApplication::loadPlugins()
 {
   int count;
   char *cfgPlugins;
-  
+  GPluginInitParams params;
+
+  params.errorContext = ectx;
+  params.config = cfg;
+  params.menu = &menuStruct;
+
   GNUNET_GC_get_configuration_value_string(cfg, "GNUNET-QT", "PLUGINS", "about general fs stats", &cfgPlugins);
   QStringList plugins = QString(cfgPlugins).split(QRegExp("\\s+"));
   count = plugins.count();
-  
+
   while(count)
   {
     QWidget *child;
     QString strPlugin = plugins.takeFirst();
-    
-    child = loader.load(strPlugin, cfg, ectx);
+
+    child = loader.load(strPlugin, &params);
     if (child)
        wnd.addApplication(child, child->windowIcon(), child->windowTitle());
 
     count--;
   }
-  
+
   GNUNET_free(cfgPlugins);
 }
 
 void GApplication::showWindow()
 {
+  wnd.loadMenuStruct(menuStruct);
   wnd.show();
 }
 
@@ -227,7 +278,27 @@ GMainWindow *GApplication::getWindow()
   return &wnd;
 }
 
-int main(int argc, char * const *argv)
+#if defined(Q_OS_WIN)
+bool GApplication::winEventFilter(MSG *msg, long *result)
+{
+  if (msg->message == WM_CLOSE)
+  {
+    *result = 0;
+    wnd.setVisible(false);
+    return true;
+  }
+
+  return false;
+}
+#endif
+
+int
+#if defined(Q_OS_WIN)
+ gn_main
+#else
+ main
+#endif
+(int argc, char * const *argv)
 {
   int ret = 0;
   QTranslator transl;
@@ -256,8 +327,11 @@ int main(int argc, char * const *argv)
       "gnunet-qt" + QDir::separator() + "locale" + QDir::separator());
   app->installTranslator(&transl);
 
+  app->addLibraryPath(app->applicationDirPath() + "/../lib/GNUnet/");
+
+  app->setupMenuStruct();
   app->loadPlugins();
-	app->showWindow();
+  app->showWindow();
 
 #if defined(Q_OS_WIN)
   if (!debug_mode)
@@ -267,11 +341,54 @@ int main(int argc, char * const *argv)
   ret = app->exec();
 
   delete app;
-  
+
   GNUNET_GE_free_context(ectx);
   GNUNET_fini(stdECtx, cfg);
-  
+
 	return ret;
 }
 
+
+#if defined(Q_OS_WIN)
+int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
+{
+  int argc, ret;
+  char **argv, *idx, *line, *start, mod[4097];
+  bool quoted, esc;
+
+  argc = 0;
+  quoted = esc = 0;
+  line = strdup(lpCmdLine);
+  argv = (char **) malloc(sizeof(char *) * 2);
+
+  GetModuleFileNameA(NULL, mod, 4096);
+  argv[0] = mod;
+  argv[1] = NULL;
+  start = line;
+
+  for (idx = line; *idx != 0; idx++)
+  {
+    if (*idx == '"' && (idx == line || *(idx-1) != '\\'))
+      quoted = !quoted;
+
+    if (!quoted && (*idx == '\t' || *idx == ' '))
+    {
+      argc++;
+      argv = (char **) realloc(argv, (argc + 2) * sizeof(char *));
+      argv[argc] = start;
+      argv[argc + 1] = NULL;
+      *idx = 0;
+      start = idx + 1;
+    }
+  }
+
+  ret = gn_main(argc + 1, (char * const *) argv);
+  free(argv);
+  free(line);
+
+  return ret;
+}
+#endif
+
+
 /* end of main.cc */
