@@ -34,9 +34,16 @@
 #include "../common/event.h"
 #include "../common/eventDispatcher.h"
 #include "../common/textEditor.h"
+#include "../common/plugin.h"
+#include "../common/pluginLoader.h"
 
 #define GNUNETQT_ASSERT(cond)  do { if (! (cond)) gnunet_qt_assert_quit(__FILE__, __LINE__); } while(0);
 
+// FIXME
+#ifndef _
+  #define _
+#endif
+
 #ifdef Q_OS_WIN32
   #define GNUNETQT_API __declspec(dllexport)
 #else
@@ -47,36 +54,6 @@ void gnunet_qt_assert_quit(char *file, int line);
 
 typedef QList<int> GIntList;
 
-typedef struct
-{
-  class QLibrary *lib;
-  class GPlugin *wnd;
-} GPluginSpec;
-
-class GPlugin: public QWidget
-{
-  Q_OBJECT
-  
-public:  
-signals:
-  int setStatusText(const QPixmap &icon, const QString strStatus);
-  int setNetworkStatus(const QPixmap &icon, const QString strStatus);
-};
-
-class GPluginLoader : public QObject
-{
-  Q_OBJECT
-  
-public:
-  ~GPluginLoader();
-  GPlugin *load(const QString &strName, struct GNUNET_GC_Configuration *config,
-    struct GNUNET_GE_Context *errorContext);
-  void unloadAll();
-  
-protected:
-  QList<GPluginSpec> plugins;
-};
-
 class GString : public QString
 {
 public:
@@ -108,11 +85,6 @@ protected:
   char *cstr;
 };
 
-typedef GPlugin *(*InitPlugin) (const struct GNUNET_GC_Configuration *config,
-  const struct GNUNET_GE_Context *errorContext);
-typedef void (*ShutdownPlugin) (GPlugin *plugin);
-
-
 #endif /*GNUNET_QT_COMMON_H_*/
 
 /* end of gnunet_qt_common.h */
