@@ -311,7 +311,7 @@ void GAboutPlugin::linkHandler(const QUrl &link)
 extern "C"
 {
 
-GNUNETQT_API GPlugin *init_plugin(struct GNUNET_GC_Configuration *config,
+GNUNETQT_API GPlugin *init_about(struct GNUNET_GC_Configuration *config,
   struct GNUNET_GE_Context *errorContext)
 {
   Q_UNUSED(config)
@@ -320,7 +320,7 @@ GNUNETQT_API GPlugin *init_plugin(struct GNUNET_GC_Configuration *config,
   return new GAboutPlugin();
 }
 
-GNUNETQT_API void shutdown_plugin(GPlugin *plugin)
+GNUNETQT_API void shutdown_about(GPlugin *plugin)
 {
   delete (GAboutPlugin *) plugin;
 }
