@@ -37,6 +37,12 @@ class GAboutPlugin : public GPlugin, public Ui::WndAbout
   
 public:
   GAboutPlugin();
+
+// unused
+signals:
+    int setStatusText(const QPixmap &icon, const QString strStatus);
+    int setNetworkStatus(const QPixmap &icon, const QString strStatus);
+
 protected:
   QString header();
   void welcome();
