@@ -258,7 +258,7 @@ void GFSDownloadController::clear()
       QStandardItem *item;
       
       item = downloadModel.item(idx.row());
-      GNUNET_FSUI_download_stop(fs->context(), (struct GNUNET_FSUI_DownloadList *)
+      GNUNET_FSUI_download_stop((struct GNUNET_FSUI_DownloadList *)
         item->data(Qt::UserRole).value<void *>());      
     }
     else
@@ -268,8 +268,8 @@ void GFSDownloadController::clear()
 
 void GFSDownloadController::cancel(struct GNUNET_FSUI_DownloadList *handle)
 {
-  GNUNET_FSUI_download_abort(fs->context(), handle);
-  GNUNET_FSUI_download_stop(fs->context(), handle);
+  GNUNET_FSUI_download_abort(handle);
+  GNUNET_FSUI_download_stop(handle);
 }
 
 QAbstractItemModel *GFSDownloadController::model()
