@@ -27,6 +27,8 @@
 #include "ecrsuri.h"
 #include "gnunet_qt_common.h"
 
+#include <GNUnet/gnunet_namespace_lib.h>
+
 GFSEcrsUri::GFSEcrsUri()
 {
   ecrsUri = NULL;
@@ -114,6 +116,19 @@ QString GFSEcrsUri::toString()
   return strRet;
 }
 
+QString GFSEcrsUri::toDisplayString(struct GNUNET_GC_Configuration *cfg,
+    struct GNUNET_GE_Context *ectx)
+{
+  if (GNUNET_ECRS_uri_test_ksk(ecrsUri))
+    return GNUNET_ECRS_ksk_uri_to_human_readable_string(ecrsUri);
+  else
+    return GNUNET_NS_sks_uri_to_human_readable_string(ectx, cfg, ecrsUri);
+
+  GNUNET_GE_BREAK (ectx, 0);
+
+  return "";
+}
+
 QString GFSEcrsUri::serialized()
 {
   char *str;
