@@ -41,6 +41,8 @@ public:
   
   const GNUNET_ECRS_URI *uri();
   QString toString();
+  QString toDisplayString(struct GNUNET_GC_Configuration *cfg,
+      struct GNUNET_GE_Context *ectx);
   QString serialized();
   
   bool operator==(const GFSEcrsUri &rvalue);
