@@ -42,9 +42,13 @@
 #include "fs-search.h"
 #include "ecrsuri.h"
 #include "searchController.h"
+#include "downloadController.h"
 
 static void *fsuiEventProcessor(void *cls, const GNUNET_FSUI_Event *event)
 {
+  void *ret;
+  int i;
+  
   GFSSearchController *searchCntrl = ((GFSPlugin *) cls)->searchController();
   GFSUploadController *uploadCntrl = ((GFSPlugin *) cls)->uploadController();
   GFSDownloadController *downloadCntrl = ((GFSPlugin *) cls)->downloadController();
@@ -59,23 +63,34 @@ static void *fsuiEventProcessor(void *cls, const GNUNET_FSUI_Event *event)
       searchCntrl->result((GFSSearchInfo *) event->data.SearchResult.sc.cctx,
         &event->data.SearchResult.fi);
       break;
-    case GNUNET_FSUI_search_error:
-      GNUNET_GE_LOG(errorContext, (GNUNET_GE_KIND) (GNUNET_GE_ERROR | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
-        event->data.SearchError.message);
     case GNUNET_FSUI_search_aborted:
-    case GNUNET_FSUI_search_completed:
     case GNUNET_FSUI_search_suspended:
       searchCntrl->state((GFSSearchInfo *) event->data.SearchResult.sc.cctx,
         event->type);
       break;
     case GNUNET_FSUI_search_resumed:
-      return searchCntrl->started(event->data.SearchResumed.sc.pos,
+      ret = searchCntrl->started(event->data.SearchResumed.sc.pos,
         event->data.SearchResumed.searchURI,
         event->data.SearchResumed.fisSize,
         event->data.SearchResumed.fis);
+      for (i=0; i < event->data.SearchResumed.fisSize; i++)
+        searchCntrl->update((GFSSearchInfo *) ret,
+                         &event->data.SearchResumed.fis[i],
+                         event->data.SearchResumed.searchURI,
+                         event->data.SearchResumed.availability_rank[i],
+                         event->data.SearchResumed.availability_certainty[i],
+                         event->data.SearchResumed.applicability_rank[i]);
+      return ret;
     case GNUNET_FSUI_search_stopped:
       searchCntrl->stopped((GFSSearchInfo *) event->data.SearchStopped.sc.cctx);
       break;
+    case GNUNET_FSUI_search_update:
+      searchCntrl->update((GFSSearchInfo *) event->data.SearchUpdate.sc.cctx,
+          &event->data.SearchUpdate.fi, event->data.SearchUpdate.searchURI,
+          event->data.SearchUpdate.availability_rank,
+          event->data.SearchUpdate.availability_certainty,
+          event->data.SearchUpdate.applicability_rank);
+      break;
     case GNUNET_FSUI_upload_started:
       return uploadCntrl->started((QPersistentModelIndex *) event->data.UploadStarted.uc.pcctx,
         event->data.UploadStarted.filename, event->data.UploadStarted.total, 0);
@@ -145,6 +160,7 @@ GFSPlugin::GFSPlugin(struct GNUNET_GC_Configuration *config,
   qRegisterMetaType<QModelIndex>("QModelIndex");
   qRegisterMetaType<Qt::Orientation>("Qt::Orientation");
   qRegisterMetaType<QModelIndexList>("QModelIndexList");
+  qRegisterMetaType<GRanking>("GRanking");
 
   cfg = config;
   ectx = errorContext;
@@ -216,7 +232,7 @@ void GFSPlugin::openDownloadClicked()
   {
     QModelIndex idx = *it;
     
-    if (idx.column() == 4)
+    if (idx.column() == GFSDownloadController::COL_DST_PATH)
     {
       QString name;
 
@@ -276,12 +292,11 @@ void GFSPlugin::searchClicked()
   else
     uri = GNUNET_ECRS_keyword_string_to_uri(ectx, strSearch.toLocal8Bit().data());
 
-  if (uri == NULL)
+  if (uri == NULL || searchCntrl->isActive(uri))
     return;
   
   /* Start search */
-  // @TODO: FIXME: INT_MAX
-  GNUNET_FSUI_search_start(fsuiContext, spinSearchAnon->value(), INT_MAX, INT_MAX,
+  GNUNET_FSUI_search_start(fsuiContext, spinSearchAnon->value(), 
     uri);
 
   GNUNET_ECRS_uri_destroy(uri);
@@ -344,7 +359,7 @@ bool GFSPlugin::event(QEvent *e)
       /* first tab is already used. Create a new one. */
       tab = new GFSSearch(this, tabResults);
       tab->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-      tabResults->addTab(tab, info->uri.toString());
+      tabResults->addTab(tab, info->uri.toDisplayString(cfg, ectx));
       tabIdx = tabResults->count() - 1;
     }
     else
@@ -506,7 +521,6 @@ QString GFSPlugin::fsuiState(GNUNET_FSUI_EventType type)
     case GNUNET_FSUI_unindex_stopped:
       return tr("stopped");
 
-    case GNUNET_FSUI_search_completed:
     case GNUNET_FSUI_download_completed:
     case GNUNET_FSUI_upload_completed:
     case GNUNET_FSUI_unindex_completed:
@@ -518,7 +532,6 @@ QString GFSPlugin::fsuiState(GNUNET_FSUI_EventType type)
     case GNUNET_FSUI_unindex_aborted:
       return tr("aborted");
 
-    case GNUNET_FSUI_search_error:
     case GNUNET_FSUI_download_error:
     case GNUNET_FSUI_upload_error:
     case GNUNET_FSUI_unindex_error:
@@ -543,13 +556,13 @@ QString GFSPlugin::fsuiState(GNUNET_FSUI_EventType type)
 
 extern "C"
 {
-  GNUNETQT_API GPlugin *init_plugin(struct GNUNET_GC_Configuration *config,
+  GNUNETQT_API GPlugin *init_fs(struct GNUNET_GC_Configuration *config,
     struct GNUNET_GE_Context *errorContext)
   {
     return new GFSPlugin(config, errorContext);
   }
   
-  GNUNETQT_API void shutdown_plugin(GPlugin *plugin)
+  GNUNETQT_API void shutdown_fs(GPlugin *plugin)
   {
     delete plugin;
   }
