@@ -83,6 +83,11 @@ protected:
   struct GNUNET_GE_Context *ectx;
   class GSearchItemDelegate *searchItemDelegate;
 
+public:  
+signals:
+  int setStatusText(const QPixmap &icon, const QString strStatus);
+  int setNetworkStatus(const QPixmap &icon, const QString strStatus);
+
 protected slots:
   void searchClicked();
   void chooseClicked();
