@@ -59,7 +59,12 @@ public:
   void stopped(GFSSearchInfo *info);
   void downloadCompleted(QPersistentModelIndex &idx, GString file);
   void state(GFSSearchInfo *info, GNUNET_FSUI_EventType event);
+  void update(GFSSearchInfo *info, const GNUNET_ECRS_FileInfo *finfo,
+      const struct GNUNET_ECRS_URI *searchURI, int avail, unsigned int cert,
+      unsigned int relevance);
 
+  bool isActive(GFSEcrsUri uri);
+  
   static void addSearchResult(QStandardItemModel *model, QModelIndex parent,
     const GNUNET_ECRS_FileInfo *info);
 protected slots:
