@@ -33,7 +33,8 @@
 #include <GNUnet/gnunet_ecrs_lib.h>
 #include "uploadController.h"
 
-static int getKeyword(const char *data, void *list)
+static int getKeyword(const char *data, int,
+		      void *list)
 {
   ((QStringList *) list)->append(QString::fromUtf8(data));
   
@@ -88,7 +89,7 @@ bool GFSUploadController::start(QWidget *parent, const QString &strPath,
 {
   GFSMetaData metaData;
   QStringList keywords;
-  struct GNUNET_ECRS_MetaData *meta;
+  struct GNUNET_MetaData *meta;
   QFileInfo fileInfo;
   struct GNUNET_ECRS_URI *keywordUri;
   int thumbSize;
@@ -111,7 +112,7 @@ bool GFSUploadController::start(QWidget *parent, const QString &strPath,
   }
 
   // extract meta data
-  meta = GNUNET_ECRS_meta_data_create();
+  meta = GNUNET_meta_data_create();
   if (!meta)
   {
       GNUNET_GE_LOG(fs->errorContext(), (GNUNET_GE_KIND) (GNUNET_GE_ERROR | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
@@ -119,16 +120,16 @@ bool GFSUploadController::start(QWidget *parent, const QString &strPath,
       return false;
   }
   
-  GNUNET_ECRS_meta_data_extract_from_file(fs->errorContext(), meta, strPath.toLocal8Bit().data(), extractors);
-  GNUNET_ECRS_meta_data_get_contents(meta, &meta_data_get_by_type, &metaData);
-  thumbSize = GNUNET_ECRS_meta_data_get_thumbnail(meta, &thumb);
+  GNUNET_meta_data_extract_from_file(fs->errorContext(), meta, strPath.toLocal8Bit().data(), extractors);
+  GNUNET_meta_data_get_contents(meta, &meta_data_get_by_type, &metaData);
+  thumbSize = GNUNET_meta_data_get_thumbnail(meta, &thumb);
   if (thumbSize)
     metaData.insert(EXTRACTOR_THUMBNAIL_DATA, QByteArray((const char *) thumb,
       thumbSize));
   
   // Keywords
-  keywordUri = GNUNET_ECRS_meta_data_to_uri(meta);
-  GNUNET_ECRS_meta_data_destroy(meta);
+  keywordUri = GNUNET_meta_data_to_uri(meta);
+  GNUNET_meta_data_destroy(meta);
   if (keywordUri)
   {
     GNUNET_ECRS_uri_get_keywords_from_ksk(keywordUri, getKeyword, &keywords);
@@ -158,9 +159,9 @@ bool GFSUploadController::start(QWidget *parent, const QString &strPath,
     long idx;
     
     // prepare meta data
-    meta = GNUNET_ECRS_meta_data_create();
+    meta = GNUNET_meta_data_create();
     for(it = metaData.begin(); it != metaData.end(); it++)
-      GNUNET_ECRS_meta_data_insert(meta, it.key(), it->data());
+      GNUNET_meta_data_insert(meta, it.key(), it->data());
 
     // prepare global URI
     globalURI = GNUNET_ECRS_string_to_uri(fs->errorContext(), GNUNET_ECRS_URI_PREFIX
@@ -178,7 +179,8 @@ bool GFSUploadController::start(QWidget *parent, const QString &strPath,
       idx--;
     }
     
-    keywordURI = GNUNET_ECRS_keyword_strings_to_uri((const char **) keys);
+    keywordURI = GNUNET_ECRS_keyword_command_line_to_uri(NULL, keywords.count(),
+							 (const char **) keys);
     
     // upload
     handle = GNUNET_FSUI_upload_start(fs->context(), strPath.toLocal8Bit(),
@@ -195,7 +197,7 @@ bool GFSUploadController::start(QWidget *parent, const QString &strPath,
     }
     delete [] keys;
     
-    GNUNET_ECRS_meta_data_destroy(meta);
+    GNUNET_meta_data_destroy(meta);
     GNUNET_ECRS_uri_destroy(globalURI);
     GNUNET_ECRS_uri_destroy(keywordURI);
     
