@@ -183,13 +183,13 @@ void GGeneralPlugin::running(bool isRunning)
 extern "C"
 {
   
-  GNUNETQT_API GPlugin *init_plugin(struct GNUNET_GC_Configuration *config,
+  GNUNETQT_API GPlugin *init_general(struct GNUNET_GC_Configuration *config,
     struct GNUNET_GE_Context *errorContext)
   {
     return new GGeneralPlugin(config, errorContext);
   }
   
-  GNUNETQT_API void shutdown_plugin(GPlugin *plugin)
+  GNUNETQT_API void shutdown_general(GPlugin *plugin)
   {
     delete (GGeneralPlugin *) plugin;
   }
