@@ -48,8 +48,8 @@ GStatsPlugin::GStatsPlugin(struct GNUNET_GC_Configuration *config,
   this->errorContext = errorContext;
   statsThread = new GStatsThread(config, errorContext);
   
-  connect(statsThread, SIGNAL(stat(const char *, qulonglong)), this,
-    SLOT(processStat(const char *, qulonglong)), Qt::DirectConnection);
+  connect(statsThread, SIGNAL(stat(QString, qulonglong)), this,
+    SLOT(processStat(QString, qulonglong)), Qt::QueuedConnection);
   
   treeStats->setColumnWidth(0, 550);
   statsThread->start();
@@ -58,19 +58,17 @@ GStatsPlugin::GStatsPlugin(struct GNUNET_GC_Configuration *config,
 GStatsPlugin::~GStatsPlugin()
 {
   statsThread->stop();
-  if (!statsThread->wait(200))
+  if (!statsThread->wait(1000))
     statsThread->terminate();
 
   delete statsThread;
 }
 
-void GStatsPlugin::processStat(const char *name, qulonglong value)
+void GStatsPlugin::processStat(QString strName, qulonglong value)
 {
     int count;
     bool found;
-    QString strName;
-    
-    strName = QString::fromUtf8(name);
+
     found = false;
     
     /* populate statistics tree */
@@ -103,7 +101,7 @@ void GStatsPlugin::processStat(const char *name, qulonglong value)
     if (strName == "# of connected peers")
     {
       QPixmap icon;
-    
+      
       if (value == 0)
         icon.load(":/pixmaps/network-status-offline.png");
       else
@@ -119,13 +117,13 @@ void GStatsPlugin::processStat(const char *name, qulonglong value)
 extern "C"
 {
   
-  GNUNETQT_API GPlugin *init_plugin(struct GNUNET_GC_Configuration *config,
+  GNUNETQT_API GPlugin *init_stats(struct GNUNET_GC_Configuration *config,
     struct GNUNET_GE_Context *errorContext)
   {
     return new GStatsPlugin(config, errorContext);
   }
   
-  GNUNETQT_API void shutdown_plugin(GPlugin *plugin)
+  GNUNETQT_API void shutdown_stats(GPlugin *plugin)
   {
     delete (GStatsPlugin *) plugin;
   }
